/*
    RGB, Levels and Saturation Adjustment Filter for VirtualDub.
			Based on Red/Green/Blue (RGB) Adjustment
			Filter for VirtualDub.(1.0) by Donald A. Graft
    Copyright (C) 1999-2000 Donald A. Graft
	Copyright (C) 2004 Eugene Khoroshavin

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	The author can be contacted at:
	Eugene Khoroshavin
	fdump@mail.ru
*/

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <crtdbg.h>
#include <math.h>
#include <stdlib.h>

#include "ScriptInterpreter.h"
#include "ScriptError.h"
#include "ScriptValue.h"

#include "resource.h"
#include "filter.h"

///////////////////////////////////////////////////////////////////////////

int RunProc(const FilterActivation *fa, const FilterFunctions *ff);
int StartProc(FilterActivation *fa, const FilterFunctions *ff);
int EndProc(FilterActivation *fa, const FilterFunctions *ff);
long ParamProc(FilterActivation *fa, const FilterFunctions *ff);
int InitProc(FilterActivation *fa, const FilterFunctions *ff);
int ConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd);
void StringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str);
void ScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc);
bool FssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen);

///////////////////////////////////////////////////////////////////////////
const char * rtxt = "Red";
const char * gtxt = "Green";
const char * btxt = "Blue";
const char * dtxt = "Dark";
const char * mtxt = "Middle";
const char * ltxt = "Light";
bool rgbFlg = false;
bool lvFlg = false;
bool satFlg = false;
typedef struct MyFilterData {
	IFilterPreview		*ifp;
	int					redD;
	int					greenD;
	int					blueD;
	int					redM;
	int					greenM;
	int					blueM;
	int					redL;
	int					greenL;
	int					blueL;
	int					flgs;
	int					lvD;
	int					lvM;
	int					lvL;
	int					satD;
	int					satM;
	int					satL;
} MyFilterData;

void setFlg (MyFilterData *mfd);

bool FssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	_snprintf(buf, buflen, "Config(%d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d)",
		mfd->redD,
		mfd->greenD,
		mfd->blueD,
		mfd->redM,
		mfd->greenM,
		mfd->blueM,
		mfd->redL,
		mfd->greenL,
		mfd->blueL,
		mfd->flgs,
		mfd->lvD,
		mfd->lvM,
		mfd->lvL,
		mfd->satD,
		mfd->satM,
		mfd->satL);

	return true;
}

void ScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc) {
	FilterActivation *fa = (FilterActivation *)lpVoid;
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	mfd->redD			= argv[0].asInt();
	mfd->greenD			= argv[1].asInt();
	mfd->blueD			= argv[2].asInt();
	mfd->redM			= argv[3].asInt();
	mfd->greenM			= argv[4].asInt();
	mfd->blueM			= argv[5].asInt();
	mfd->redL			= argv[6].asInt();
	mfd->greenL			= argv[7].asInt();
	mfd->blueL			= argv[8].asInt();
	mfd->flgs			= argv[9].asInt();
	mfd->lvD			= argv[10].asInt();
	mfd->lvM			= argv[11].asInt();
	mfd->lvL			= argv[12].asInt();
	mfd->satD			= argv[13].asInt();
	mfd->satM			= argv[14].asInt();
	mfd->satL			= argv[15].asInt();
	setFlg(mfd);
}

ScriptFunctionDef func_defs[]={
	{ (ScriptFunctionPtr)ScriptConfig, "Config", "0iiiiiiiiiiiiiiii" },
	{ NULL },
};

CScriptObject script_obj={
	NULL, func_defs
};

struct FilterDefinition filterDef_tutorial = {

	NULL, NULL, NULL,		// next, prev, module
	"RGB/Levels/Saturation Balance(1.03)",	// name
	"Adjust RGB, Levels and Saturation.",
							// desc
	"Eugene Khoroshavin", 		// maker
	NULL,					// private_data
	sizeof(MyFilterData),	// inst_data_size

	InitProc,				// initProc
	NULL,					// deinitProc
	RunProc,				// runProc
	NULL,					// paramProc
	ConfigProc, 			// configProc
	StringProc, 			// stringProc
	StartProc,				// startProc
	EndProc,				// endProc

	&script_obj,			// script_obj
	FssProc,				// fssProc

};

int StartProc(FilterActivation *fa, const FilterFunctions *ff)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	return 0;
}

int EndProc(FilterActivation *fa, const FilterFunctions *ff)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	return 0;
}

///////////////////////////////////////////////////////////////////////////

int RunProc(const FilterActivation *fa, const FilterFunctions *ff)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	const PixDim	width = fa->src.w;
	const PixDim	height = fa->src.h;
	const long u22 = -16384;
	Pixel32 *src, *dst;
	int x, y;
	long r, g, b, u, up;
	src = fa->src.data;
	dst = fa->dst.data;
	for (y = 0; y < height; y++)
	{
		for (x = 0; x < width; x++)
		{
			r = ((src[x] & 0xff0000) >> 16);
			g = ((src[x] & 0xff00) >> 8);
			b = (src[x] & 0xff);
			if (mfd->flgs & 0x02) {
				if (lvFlg) {
					u = r + g + b;
					up = u/3;
					if (up < 128) {
						long inc = (mfd->lvD * (up * up - 16384) + mfd->lvM * (up * up - 256 * up)) / u22;
						r +=  inc;
						g +=  inc;
						b +=  inc;}
						else {
						long inc = (mfd->lvM * (up * up - 256 * up) + mfd->lvL * (up * up - 512 * up + 49152)) / u22;
						r += inc;
						g += inc;
						b += inc;}
					if (r < 0) r = 0;
					if (r > 255) r = 255;
					if (g < 0) g = 0;
					if (g > 255) g = 255;
					if (b < 0) b = 0;
					if (b > 255) b = 255;
				}

				if (rgbFlg) {
					u = r + g + b;
					up = u/3;
					if (up < 128) { 
						long inca = (up * up - 16384);
						long incb = (up * up - 256 * up);
						r += ((mfd->redD * inca + mfd->redM * incb) / u22);
						g += ((mfd->greenD * inca + mfd->greenM * incb) / u22);
						b += ((mfd->blueD * inca + mfd->blueM * incb) / u22);}
					else {
						long inca = (up * up - 256 * up);
						long incb = (up * up - 512 * up + 49152);
						r += ((mfd->redM * inca + mfd->redL * incb) / u22);
						g += ((mfd->greenM * inca + mfd->greenL * incb) / u22);
						b += ((mfd->blueM * inca + mfd->blueL * incb) / u22);}
					if (r < 0) r = 0;
					if (r > 255) r = 255;
					if (g < 0) g = 0;
					if (g > 255) g = 255;
					if (b < 0) b = 0;
					if (b > 255) b = 255;

					up = r + g + b;
					if ((mfd->flgs & 0x01) && (up > 0))
					{
						u = u - up;
						r += (u * r)/up;
						g += (u * g)/up;
						b += (u * b)/up;
						if (r < 0) r = 0;
						if (r > 255) r = 255;
						if (g < 0) g = 0;
						if (g > 255) g = 255;
						if (b < 0) b = 0;
						if (b > 255) b = 255;
					}
				}

				if (satFlg) {
					u = r + g + b;
					up = u/3;
					if (up < 128) {
						long inc =  (mfd->satD * (up * up - 16384) + mfd->satM * (up * up - 256 * up)) / u22;
						inc = 100 + inc * 2 + unsigned long(inc);
						if (inc < 0) inc = 0;
						r = up + ((r - up)*inc)/100;
						g = up + ((g - up)*inc)/100;
						b = up + ((b - up)*inc)/100;}
						else {
						long inc = (mfd->satM * (up * up - 256 * up) + mfd->satL * (up * up - 512 * up + 49152)) / u22;
						inc = 100 + inc * 2 + unsigned long(inc);
						if (inc < 0) inc = 0;
						r = up + ((r - up)*inc)/100;
						g = up + ((g - up)*inc)/100;
						b = up + ((b - up)*inc)/100;}
					if (r < 0) r = 0;
					if (r > 255) r = 255;
					if (g < 0) g = 0;
					if (g > 255) g = 255;
					if (b < 0) b = 0;
					if (b > 255) b = 255;
				}
			}
			else {
				if (lvFlg) {
					u = r + g + b;
					up = u/3;
					if (up < 128) {
						long inc = ((mfd->lvD * (128-up)) + (mfd->lvM * up))/127;
						r +=  inc;
						g +=  inc;
						b +=  inc;}
						else {
						long inc = ((mfd->lvM * (256-up)) + (mfd->lvL * (up-127)))/128;
						r += inc;
						g += inc;
						b += inc;}
					if (r < 0) r = 0;
					if (r > 255) r = 255;
					if (g < 0) g = 0;
					if (g > 255) g = 255;
					if (b < 0) b = 0;
					if (b > 255) b = 255;
				}

				if (rgbFlg) {
					u = r + g + b;
					up = u/3;
					if (up < 128) { 
						r += ((mfd->redD * (128-up)) + (mfd->redM * up))/127;
						g += ((mfd->greenD * (128-up)) + (mfd->greenM * up))/127;
						b += ((mfd->blueD * (128-up)) + (mfd->blueM * up))/127;}
						else {
						r += ((mfd->redM * (256-up)) + (mfd->redL * (up-127)))/128;
						g += ((mfd->greenM * (256-up)) + (mfd->greenL * (up-127)))/128;
						b += ((mfd->blueM * (256-up)) + (mfd->blueL * (up-127)))/128;}
					if (r < 0) r = 0;
					if (r > 255) r = 255;
					if (g < 0) g = 0;
					if (g > 255) g = 255;
					if (b < 0) b = 0;
					if (b > 255) b = 255;

					up = r + g + b;
					if ((mfd->flgs & 0x01) && (up > 0))
					{
						u = u - up;
						r += (u * r)/up;
						g += (u * g)/up;
						b += (u * b)/up;
						if (r < 0) r = 0;
						if (r > 255) r = 255;
						if (g < 0) g = 0;
						if (g > 255) g = 255;
						if (b < 0) b = 0;
						if (b > 255) b = 255;
					}
				}

				if (satFlg) {
					u = r + g + b;
					up = u/3;
					if (up < 128) {
						long inc = 12700 + mfd->satD * (128-up) + mfd->satM * up;
						r = up + ((r - up)*inc)/12700;
						g = up + ((g - up)*inc)/12700;
						b = up + ((b - up)*inc)/12700;}
						else {
						long inc = 12800 + mfd->satM * (256-up) + mfd->satL * (up-127);
						r = up + ((r - up)*inc)/12800;
						g = up + ((g - up)*inc)/12800;
						b = up + ((b - up)*inc)/12800;}
					if (r < 0) r = 0;
					if (r > 255) r = 255;
					if (g < 0) g = 0;
					if (g > 255) g = 255;
					if (b < 0) b = 0;
					if (b > 255) b = 255;
				}
			}
			dst[x] = ((int) r << 16) | ((int) g << 8) | ((int) b);			 
		}
		src	= (Pixel *)((char *)src + fa->src.pitch);
		dst	= (Pixel *)((char *)dst + fa->dst.pitch);
	}
	return 0;
}

extern "C" int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat);
extern "C" void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff);

static FilterDefinition *fd_tutorial;

int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
	if (!(fd_tutorial = ff->addFilter(fm, &filterDef_tutorial, sizeof(FilterDefinition))))
		return 1;

	vdfd_ver = VIRTUALDUB_FILTERDEF_VERSION;
	vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;

	return 0;
}

void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff) {
	ff->removeFilter(fd_tutorial);
}

int InitProc(FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	mfd->redD = 0;
	mfd->greenD = 0;
	mfd->blueD = 0;
	mfd->redM = 0;
	mfd->greenM = 0;
	mfd->blueM = 0;
	mfd->redL = 0;
	mfd->greenL = 0;
	mfd->blueL = 0;
	mfd->flgs = 0x01;
	mfd->lvD = 0;
	mfd->lvM = 0;
	mfd->lvL = 0;
	mfd->satD = 0;
	mfd->satM = 0;
	mfd->satL = 0;
	return 0;
}

BOOL CALLBACK ConfigDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	MyFilterData *mfd = (MyFilterData *)GetWindowLong(hdlg, DWL_USER);
	int mPr=0;

	switch(msg) {
		case WM_INITDIALOG:
			SetWindowLong(hdlg, DWL_USER, lParam);
			mfd = (MyFilterData *)lParam;
			HWND hWnd;

			SetDlgItemText(hdlg, IDC_RTEXT, rtxt);
			SetDlgItemText(hdlg, IDC_GTEXT, gtxt);
			SetDlgItemText(hdlg, IDC_BTEXT, btxt);
			CheckRadioButton(hdlg,IDC_DARK, IDC_SAT, IDC_MIDDLE);
			hWnd = GetDlgItem(hdlg, IDC_RED);
			SendMessage(hWnd, TBM_SETRANGE, (WPARAM)TRUE, MAKELONG(-100, 100));
			SendMessage(hWnd, TBM_SETTICFREQ, 10 , 0);
			SendMessage(hWnd, TBM_SETPOS, (WPARAM)TRUE, mfd->redM);
			SetDlgItemInt(hdlg, IDC_REDVAL, mfd->redM, TRUE);
			hWnd = GetDlgItem(hdlg, IDC_GREEN);
			SendMessage(hWnd, TBM_SETRANGE, (WPARAM)TRUE, MAKELONG(-100, 100));
			SendMessage(hWnd, TBM_SETTICFREQ, 10, 0);
			SendMessage(hWnd, TBM_SETPOS, (WPARAM)TRUE, mfd->greenM);
			SetDlgItemInt(hdlg, IDC_GREENVAL, mfd->greenM, TRUE);
			hWnd = GetDlgItem(hdlg, IDC_BLUE);
			SendMessage(hWnd, TBM_SETRANGE, (WPARAM)TRUE, MAKELONG(-100, 100));
			SendMessage(hWnd, TBM_SETTICFREQ, 10, 0);
			SendMessage(hWnd, TBM_SETPOS, (WPARAM)TRUE, mfd->blueM);
			SetDlgItemInt(hdlg, IDC_BLUEVAL, mfd->blueM, TRUE);
			hWnd = GetDlgItem(hdlg, IDC_KEEPBR);
			SendMessage(hWnd, BM_SETCHECK, ((mfd->flgs & 0x01) > 0), 0);
			hWnd = GetDlgItem(hdlg, IDC_BEDSRC);
			SendMessage(hWnd, BM_SETCHECK, ((mfd->flgs & 0x02) > 0), 0);
			setFlg(mfd);
			mfd->ifp->InitButton(GetDlgItem(hdlg, IDPREVIEW));
			mfd->ifp->Toggle(hdlg);
			return TRUE;

		case WM_COMMAND:
			switch(LOWORD(wParam)) {
			case IDPREVIEW:
				mfd->ifp->Toggle(hdlg);
				break;
			case IDOK:
				EndDialog(hdlg, 0);
				return TRUE;
			case IDHELP:
				{
				char prog[256];
				char path[256];
				LPTSTR ptr;
				GetModuleFileName(NULL, prog, 255);
				GetFullPathName(prog, 255, path, &ptr);
				*ptr = 0;
				strcat(path, "plugins\\RgbLvlSat.txt");
				OutputDebugString(path);
				OutputDebugString("\n");
				strcpy(prog, "Notepad ");
				strcat(prog, path);
				WinExec(prog, SW_SHOW);
				return TRUE;
				}
			case IDCANCEL:
				EndDialog(hdlg, 1);
				return TRUE;
			case IDC_ZEROR:
				SendMessage(GetDlgItem(hdlg, IDC_RED), TBM_SETPOS, (WPARAM)TRUE, 0);
				SetDlgItemInt(hdlg, IDC_REDVAL, 0, TRUE);
				mPr=SendMessage(GetDlgItem(hdlg, IDC_DARK), BM_GETCHECK, 0, 0);
				if (mPr==BST_CHECKED) mfd->redD = 0;
				mPr=SendMessage(GetDlgItem(hdlg, IDC_MIDDLE), BM_GETCHECK, 0, 0);
				if (mPr==BST_CHECKED) mfd->redM = 0;
				mPr=SendMessage(GetDlgItem(hdlg, IDC_LIGHT), BM_GETCHECK, 0, 0);
				if (mPr==BST_CHECKED) mfd->redL = 0;
				mPr=SendMessage(GetDlgItem(hdlg, IDC_LEVELS), BM_GETCHECK, 0, 0);
				if (mPr==BST_CHECKED) mfd->lvD = 0;
				mPr=SendMessage(GetDlgItem(hdlg, IDC_SAT), BM_GETCHECK, 0, 0);
				if (mPr==BST_CHECKED) mfd->satD = 0;
				setFlg(mfd);
				mfd->ifp->RedoFrame();
				return TRUE;
			case IDC_ZEROG:
				SendMessage(GetDlgItem(hdlg, IDC_GREEN), TBM_SETPOS, (WPARAM)TRUE, 0);
				SetDlgItemInt(hdlg, IDC_GREENVAL, 0, TRUE);
				mPr=SendMessage(GetDlgItem(hdlg, IDC_DARK), BM_GETCHECK, 0, 0);
				if (mPr==BST_CHECKED) mfd->greenD = 0;
				mPr=SendMessage(GetDlgItem(hdlg, IDC_MIDDLE), BM_GETCHECK, 0, 0);
				if (mPr==BST_CHECKED) mfd->greenM = 0;
				mPr=SendMessage(GetDlgItem(hdlg, IDC_LIGHT), BM_GETCHECK, 0, 0);
				if (mPr==BST_CHECKED) mfd->greenL = 0;
				mPr=SendMessage(GetDlgItem(hdlg, IDC_LEVELS), BM_GETCHECK, 0, 0);
				if (mPr==BST_CHECKED) mfd->lvM = 0;
				mPr=SendMessage(GetDlgItem(hdlg, IDC_SAT), BM_GETCHECK, 0, 0);
				if (mPr==BST_CHECKED) mfd->satM = 0;
				setFlg(mfd);
				mfd->ifp->RedoFrame();
				return TRUE;
			case IDC_ZEROB:
				SendMessage(GetDlgItem(hdlg, IDC_BLUE), TBM_SETPOS, (WPARAM)TRUE, 0);
				SetDlgItemInt(hdlg, IDC_BLUEVAL, 0, TRUE);
				mPr=SendMessage(GetDlgItem(hdlg, IDC_DARK), BM_GETCHECK, 0, 0);
				if (mPr==BST_CHECKED) mfd->blueD = 0;
				mPr=SendMessage(GetDlgItem(hdlg, IDC_MIDDLE), BM_GETCHECK, 0, 0);
				if (mPr==BST_CHECKED) mfd->blueM = 0;
				mPr=SendMessage(GetDlgItem(hdlg, IDC_LIGHT), BM_GETCHECK, 0, 0);
				if (mPr==BST_CHECKED) mfd->blueL = 0;
				mPr=SendMessage(GetDlgItem(hdlg, IDC_LEVELS), BM_GETCHECK, 0, 0);
				if (mPr==BST_CHECKED) mfd->lvL = 0;
				mPr=SendMessage(GetDlgItem(hdlg, IDC_SAT), BM_GETCHECK, 0, 0);
				if (mPr==BST_CHECKED) mfd->satL = 0;
				setFlg(mfd);
				mfd->ifp->RedoFrame();
				return TRUE;
			case IDC_ZEROALL:
				SendMessage(GetDlgItem(hdlg, IDC_RED), TBM_SETPOS, (WPARAM)TRUE, 0);
				SetDlgItemInt(hdlg, IDC_REDVAL, 0, TRUE);
				mfd->redD = 0;
				mfd->redM = 0;
				mfd->redL = 0;
				mfd->lvD = 0;
				mfd->satD = 0;
				SendMessage(GetDlgItem(hdlg, IDC_GREEN), TBM_SETPOS, (WPARAM)TRUE, 0);
				SetDlgItemInt(hdlg, IDC_GREENVAL, 0, TRUE);
				mfd->greenD = 0;
				mfd->greenM = 0;
				mfd->greenL = 0;
				mfd->lvM = 0;
				mfd->satM = 0;
				SendMessage(GetDlgItem(hdlg, IDC_BLUE), TBM_SETPOS, (WPARAM)TRUE, 0);
				SetDlgItemInt(hdlg, IDC_BLUEVAL, 0, TRUE);
				mfd->blueD = 0;
				mfd->blueM = 0;
				mfd->blueL = 0;
				mfd->lvL = 0;
				mfd->satL = 0;
				setFlg(mfd);
				mfd->ifp->RedoFrame();
				return TRUE;
			case IDC_DARK:
				SetDlgItemText(hdlg, IDC_RTEXT, rtxt);
				SetDlgItemText(hdlg, IDC_GTEXT, gtxt);
				SetDlgItemText(hdlg, IDC_BTEXT, btxt);

				SendMessage(GetDlgItem(hdlg, IDC_RED), TBM_SETPOS, (WPARAM)TRUE, mfd->redD);
				SetDlgItemInt(hdlg, IDC_REDVAL, mfd->redD, TRUE);
				SendMessage(GetDlgItem(hdlg, IDC_GREEN), TBM_SETPOS, (WPARAM)TRUE, mfd->greenD);
				SetDlgItemInt(hdlg, IDC_GREENVAL, mfd->greenD, TRUE);
				SendMessage(GetDlgItem(hdlg, IDC_BLUE), TBM_SETPOS, (WPARAM)TRUE, mfd->blueD);
				SetDlgItemInt(hdlg, IDC_BLUEVAL, mfd->blueD, TRUE);
				return TRUE;
				//break;
			case IDC_MIDDLE:
				SetDlgItemText(hdlg, IDC_RTEXT, rtxt);
				SetDlgItemText(hdlg, IDC_GTEXT, gtxt);
				SetDlgItemText(hdlg, IDC_BTEXT, btxt);

				SendMessage(GetDlgItem(hdlg, IDC_RED), TBM_SETPOS, (WPARAM)TRUE, mfd->redM);
				SetDlgItemInt(hdlg, IDC_REDVAL, mfd->redM, TRUE);
				SendMessage(GetDlgItem(hdlg, IDC_GREEN), TBM_SETPOS, (WPARAM)TRUE, mfd->greenM);
				SetDlgItemInt(hdlg, IDC_GREENVAL, mfd->greenM, TRUE);
				SendMessage(GetDlgItem(hdlg, IDC_BLUE), TBM_SETPOS, (WPARAM)TRUE, mfd->blueM);
				SetDlgItemInt(hdlg, IDC_BLUEVAL, mfd->blueM, TRUE);
				return TRUE;
			case IDC_LIGHT:
				SetDlgItemText(hdlg, IDC_RTEXT, rtxt);
				SetDlgItemText(hdlg, IDC_GTEXT, gtxt);
				SetDlgItemText(hdlg, IDC_BTEXT, btxt);
				
				SendMessage(GetDlgItem(hdlg, IDC_RED), TBM_SETPOS, (WPARAM)TRUE, mfd->redL);
				SetDlgItemInt(hdlg, IDC_REDVAL, mfd->redL, TRUE);
				SendMessage(GetDlgItem(hdlg, IDC_GREEN), TBM_SETPOS, (WPARAM)TRUE, mfd->greenL);
				SetDlgItemInt(hdlg, IDC_GREENVAL, mfd->greenL, TRUE);
				SendMessage(GetDlgItem(hdlg, IDC_BLUE), TBM_SETPOS, (WPARAM)TRUE, mfd->blueL);
				SetDlgItemInt(hdlg, IDC_BLUEVAL, mfd->blueL, TRUE);
				return TRUE;
			case IDC_KEEPBR:{
				UINT keepb = IsDlgButtonChecked(hdlg, IDC_KEEPBR);
				mfd->flgs = (mfd->flgs & ~0x01) | keepb;
				mfd->ifp->RedoFrame();}
				return TRUE;
			case IDC_LEVELS:
				SetDlgItemText(hdlg, IDC_RTEXT, dtxt);
				SetDlgItemText(hdlg, IDC_GTEXT, mtxt);
				SetDlgItemText(hdlg, IDC_BTEXT, ltxt);

				SendMessage(GetDlgItem(hdlg, IDC_RED), TBM_SETPOS, (WPARAM)TRUE, mfd->lvD);
				SetDlgItemInt(hdlg, IDC_REDVAL, mfd->lvD, TRUE);
				SendMessage(GetDlgItem(hdlg, IDC_GREEN), TBM_SETPOS, (WPARAM)TRUE, mfd->lvM);
				SetDlgItemInt(hdlg, IDC_GREENVAL, mfd->lvM, TRUE);
				SendMessage(GetDlgItem(hdlg, IDC_BLUE), TBM_SETPOS, (WPARAM)TRUE, mfd->lvL);
				SetDlgItemInt(hdlg, IDC_BLUEVAL, mfd->lvL, TRUE);
				return TRUE;
			case IDC_SAT:
				SetDlgItemText(hdlg, IDC_RTEXT, dtxt);
				SetDlgItemText(hdlg, IDC_GTEXT, mtxt);
				SetDlgItemText(hdlg, IDC_BTEXT, ltxt);

				SendMessage(GetDlgItem(hdlg, IDC_RED), TBM_SETPOS, (WPARAM)TRUE, mfd->satD);
				SetDlgItemInt(hdlg, IDC_REDVAL, mfd->satD, TRUE);
				SendMessage(GetDlgItem(hdlg, IDC_GREEN), TBM_SETPOS, (WPARAM)TRUE, mfd->satM);
				SetDlgItemInt(hdlg, IDC_GREENVAL, mfd->satM, TRUE);
				SendMessage(GetDlgItem(hdlg, IDC_BLUE), TBM_SETPOS, (WPARAM)TRUE, mfd->satL);
				SetDlgItemInt(hdlg, IDC_BLUEVAL, mfd->satL, TRUE);
				return TRUE;
			case IDC_BEDSRC:{
				UINT keepb = IsDlgButtonChecked(hdlg, IDC_BEDSRC);
				mfd->flgs = (mfd->flgs & ~0x02) | (keepb << 1);
				mfd->ifp->RedoFrame();}
				return TRUE;
			}
			break;
		case WM_HSCROLL:
			{
				int red, green, blue;
				red = SendMessage(GetDlgItem(hdlg, IDC_RED), TBM_GETPOS, 0, 0);
				SetDlgItemInt(hdlg, IDC_REDVAL, red, TRUE);
				green = SendMessage(GetDlgItem(hdlg, IDC_GREEN), TBM_GETPOS, 0, 0);
				SetDlgItemInt(hdlg, IDC_GREENVAL, green, TRUE);
				blue = SendMessage(GetDlgItem(hdlg, IDC_BLUE), TBM_GETPOS, 0, 0);
				SetDlgItemInt(hdlg, IDC_BLUEVAL, blue, TRUE);
				mPr=SendMessage(GetDlgItem(hdlg, IDC_DARK), BM_GETCHECK, 0, 0);
				if (mPr==BST_CHECKED) { 
						mfd->redD = red;
						mfd->greenD = green;
						mfd->blueD = blue;}
				mPr=SendMessage(GetDlgItem(hdlg, IDC_MIDDLE), BM_GETCHECK, 0, 0);
				if (mPr==BST_CHECKED) {
						mfd->redM = red;
						mfd->greenM = green;
						mfd->blueM = blue;}
				mPr=SendMessage(GetDlgItem(hdlg, IDC_LIGHT), BM_GETCHECK, 0, 0);
				if (mPr==BST_CHECKED) {
						mfd->redL = red;
						mfd->greenL = green;
						mfd->blueL = blue;}
				mPr=SendMessage(GetDlgItem(hdlg, IDC_LEVELS), BM_GETCHECK, 0, 0);
				if (mPr==BST_CHECKED) {
						mfd->lvD = red;
						mfd->lvM = green;
						mfd->lvL = blue;}
				mPr=SendMessage(GetDlgItem(hdlg, IDC_SAT), BM_GETCHECK, 0, 0);
				if (mPr==BST_CHECKED) {
						mfd->satD = red;
						mfd->satM = green;
						mfd->satL = blue;}
				mfd->ifp->RedoFrame();
				setFlg(mfd);
				break;
			}
	}

	return FALSE;
}

int ConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	MyFilterData mfd_old = *mfd;
	int ret;

	mfd->ifp = fa->ifp;

	if (DialogBoxParam(fa->filter->module->hInstModule, MAKEINTRESOURCE(IDD_FILTER),
		hwnd, ConfigDlgProc, (LPARAM)mfd))
	{
		*mfd = mfd_old;
		ret = TRUE;
	}
    else
	{
		ret = FALSE;
	}
	return(ret);
}

void StringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
}
 

inline void setFlg ( MyFilterData *mfd ) {
	rgbFlg =  ((mfd->redD != 0) || (mfd->greenD != 0) || (mfd->blueD != 0)
		|| (mfd->redM != 0) || (mfd->greenM != 0) || (mfd->blueM != 0)
		|| (mfd->redL != 0) || (mfd->greenL != 0) || (mfd->blueL != 0));
	lvFlg =  ((mfd->lvD != 0) || (mfd->lvM != 0) || (mfd->lvL != 0));
	satFlg = ((mfd->satD != 0) || (mfd->satM != 0) || (mfd->satL != 0));
}
